function [x,err,f] = nrndn(x0,tol,iprint)
%
%  [x,err,f] = nrndn(x0,tol,iprint)
%
% This script will perform rootfinding by the
% Newton Raphson with Numerical Derivatives method
% for a system of nonlinear algebraic equations.
%
% inputs:
% x0 = initial guess of x, column vector of length n
% tol = RMS tolerance on relative error on x, scalar
% iprint = value of 1 requests iteration information
% f(x) entered in the function "funkeval" at the bottom of this file
%
% outputs:
% 
% x = converged solution, column vector of length n
% f = RMS value of f, scalar
% err = RMS value of relative error on x, scalar
%
% example usage:
%
%  [x,err,f] = nrndn([2,2],1.0e-6,1)
%
% Author:  David Keffer 
% original date: May, 1999
% last modified date: March 22, 2013
%

%
%  maximum iterations
%
maxit = 1000;
%
%  system size
%
n = max(size(x0));
%
%  allocate variables
%
Residual = zeros(n,1); 
Jacobian = zeros(n,n);
InvJ = zeros(n,n);
dx = zeros(n,1);
x = zeros(n,1);
xold = zeros(n,1);
%
% extra variables for numerical derivatives
%
xeval = zeros(n,1);
xp = zeros(2);
fp = zeros(n,n,2);
dxcon = zeros(n,1);
%
%  initialize the size of the discretization 
% for numerical differentiation
%
dxcon(1:n) = 0.01;
%
%  initialization
%
x = x0;
err = 100.0;
iter = 0;
%
% convergence loop
%
while ( err > tol ) 
   for j = 1:1:n
   	dx(j) = min(dxcon(j)*x(j),dxcon(j));
      i2dx(j) = 1.0/(2.0*dx(j));
   end
   %
   %  evaluate function
   %
   Residual = funkeval(x);
   %
   %  set up a grid for evaluation of numerical partial derivatives
   %  evaluate function at grid points
   %
   for j = 1:1:n
      for i = 1:1:n
         xeval(i) = x(i);
      end
      xp(1) = x(j) - dx(j);
   	xp(2) = x(j) + dx(j);
      for k = 1:1:2
         xeval(j) = xp(k);
  			fp(:,j,k) = funkeval(xeval);
      end
   end
   %
   %  calculate numerical derivative
   %
   for i = 1:1:n
      for j = 1:1:n
         Jacobian(i,j) = i2dx(j)*( fp(i,j,2) - fp(i,j,1) );
      end
   end
   %
   %  evaluate new estimate of solution
   %
   xold = x;
   invJ = inv(Jacobian);
   deltax = -invJ*Residual;
   for j = 1:1:n
      x(j) = xold(j) + deltax(j);
   end
   %
   %  check for convergence
   %
   iter = iter +1;
   err = sqrt( sum(deltax.^2) /n  ) ;
   f = sqrt(sum(Residual.*Residual)/n);
   if (iprint == 1)
      fprintf (1,'iter = %4i, err = %9.2e f = %9.2e \n ', iter, err, f);
   end
   if ( iter > maxit)
      Residual
      error ('maximum number of iterations exceeded');
   end
end


function f = funkeval(x)
%
%  these two lines force a column vector of length n
%
n = max(size(x));
f = zeros(n,1);
%
%  enter the functions here
%
f(1) = x(1)^2 + x(2)^2 - 4;
f(2) = x(1)^2 - x(2) + 1;
